import IKTransaction from './BaseEvent/IKTransaction';
import IKResponse, { IKBadRequestResponse, IKSuccessResponse } from './API/IKResponse';
import IKGlobals from './IKGlobals';
//
export default class IKEventProcessor {
  constructor(event, context, config, respondWithFailures) {
    this.event = event;
    this.context = context;
    this.apiConfig = config;
    this._respondWithFailures = respondWithFailures;
  }
  async processEvent(execution, doNotDecodeMessage) {
    const resp = await this._processRawEvent(execution, doNotDecodeMessage);
    if (!this._respondWithFailures && resp && !(resp.getCode() >= 200 && resp.getCode() < 300)) throw new Error(JSON.stringify(resp.getBody()));
    else if (resp) return resp;
  }
  async _processRawEvent(execution, doNotDecodeMessage) {
    if (this.event.Records && this.event.Records.length > 0) { //safe check for empty events?
      //init transaction for all records
      return await (new IKTransaction(this.event, this.context, this.apiConfig, true /*batch processing*/)).execute(async (transaction) => {
        //Map records with decoded message when required
        const decodedRecords = this.event.Records.map((eventRecord) => (doNotDecodeMessage ? eventRecord.body : JSON.parse(eventRecord.body)));
        
        //for each available event
        const failureIDs = [];
        for (let eventRecord of decodedRecords) {
          //Call execution
          let resp = await execution(transaction, eventRecord);
          //check for failure
          if (!(resp.getCode() >= 200 && resp.getCode() < 300)) {
            //response with failures or fail hard at first
            if (this._respondWithFailures) failureIDs.push(eventRecord.messageId);
            else return resp;
          }
        }
        //not errored and loop ended - succeeded (might have failures)
        if (this._respondWithFailures) return { batchItemFailures: failureIDs.map((id) => ({ itemIdentifier: id })) };
        return IKSuccessResponse();
      });
    } else return IKBadRequestResponse(IKGlobals.ErrorResponseNoRecords, IKGlobals.ErrorCode_NoRecords); //no event to be processed?
  }
}
